@@ -13,6 +13,12 @@ class Token:
         """Called to setup tokenizer"""
         self.tokenids = {}
 
+    def get_token_at(self, doc, pos):
+        return self
+
+    def get_token(self, style):
+        return self
+        
     def re_start(self):
         """Returns regular expression to find begging of token"""
 
@@ -20,19 +26,31 @@ class Token:
         """Called when token started. Yield (pos, posto, tokenid)
         until exhausted"""
 
-    def get_tokenids(self, highlighter, tokenizer):
+    def get_tokenids(self):
         return tuple(self.tokenids.keys())
 
-    def resume_pos(self, highlighter, tokenizer, doc, pos):
+    def find_token_top(self, doc, pos):
         # Returns top of current keyword
         if 0 < pos < len(doc.styles):
-            p = doc.styles.rfindint(
-                self.get_tokenids(highlighter, tokenizer),
-                0, pos, comp_ne=True)
+            p = doc.styles.rfindint(self.get_tokenids(), 0, pos,
+                                    comp_ne=True)
             if p != -1:
                 return p + 1
         return 0
 
+    def get_prev_token(self, tokenizer, doc, pos):
+        """Find previous token"""
+
+        pos = self.find_token_top(doc, pos)
+        if pos == 0:
+            # current token is at top of the document.
+            return (None, None, None)
+
+        return tokenizer.highlighter.get_prev_token(doc, pos-1)
+
+    def resume_pos(self, highlighter, tokenizer, doc, pos):
+        return self.find_token_top(doc, pos)
+
     def assign_tokenid(self, tokenizer, stylename):
         tokenid = tokenizer.register_tokenid(self)
         self.tokenids[tokenid] = stylename
@@ -142,10 +160,26 @@ class SubTokenizer(Token):
         self.subtokenizer.prepare(self)
 
     def register_tokenid(self, tokenizer, token):
-        ret = self.tokenizer.highlighter.register_tokenid(self.tokenizer, self)
+        t = self if token else None
+        ret = self.tokenizer.highlighter.register_tokenid(self.tokenizer, t)
         self.sub_tokens[ret] = token
         return ret
 
+    def get_token(self, tokenid):
+        if tokenid in self.sub_tokens:
+            return self.sub_tokens[tokenid]
+        return self.tokenizer.highlighter.get_token(tokenid)
+
+    def get_token_at(self, doc, pos):
+        if pos < len(doc.styles):
+            style = doc.styles.getints(pos, pos + 1)[0]
+            return self.get_token(style)
+
+        return self
+
+    def get_prev_token(self, doc, pos):
+        return self.tokenizer.highlighter.get_prev_token(doc, pos)
+
     def re_start(self):
         return self.start
 
@@ -157,13 +191,19 @@ class SubTokenizer(Token):
         ret = yield from self.iter_subtokenizers(tokenizer, doc, pos, match)
         return ret
 
-    def resume_pos(self, highlighter, tokenizer, doc, pos):
+    def find_token_top(self, doc, pos):
         # Returns top of current keyword
         if 0 < pos < len(doc.styles):
-            p = doc.styles.rfindint(tuple(self.sub_tokens.keys()), 0, pos,
-                                    comp_ne=True)
-            if p > 0:
-                return highlighter.get_resume_pos(doc, p)
+            p = doc.styles.rfindint(tuple(self.sub_tokens.keys()),
+                                    0, pos+1, comp_ne=True)
+            if p != -1:
+                return p + 1
+        return 0
+
+    def resume_pos(self, highlighter, tokenizer, doc, pos):
+        ret = self.find_token_top(doc, pos)
+        if ret > 0:
+            return highlighter.get_resume_pos(doc, ret)
         return 0
 
     def get_stylename(self, tokenid):
@@ -251,6 +291,9 @@ class Tokenizer:
     def register_tokenid(self, obj):
         return self.highlighter.register_tokenid(self, obj)
 
+    def get_token(self, tokenid):
+        return self.highlighter.get_token(tokenid)
+
     def start(self, doc, pos):
         if self.re_starts:
             while True:
@@ -423,32 +466,67 @@ class Highlighter:
                 section.end = pos
                 section = section.parent
 
+    def get_prev_token(self, doc, pos):
+        """Return (tokenizer, token, pos) at pos or before pos."""
+        
+        if pos == 0:
+            return None, None, None
+
+        pos -= 1
+        while True:
+            style = doc.styles.getints(pos, pos + 1)[0]
+            if style == 0:
+                # not highlighted yet.
+                pass
+            else:
+                pair = self.tokenids.get(style, None)
+                # if pair is None, then this style invalid.
+                # May be set by tokenizer.
+                if pair:
+                    # if token is None, then token is not defined here. 
+                    # (e.g. white spaces)
+                    tokenizer, token = pair
+                    if token:
+                        return tokenizer, token, pos
+
+            pos = doc.styles.rfindint([style], 0, pos, comp_ne=True)
+            if pos == -1:
+                return None, None, None
+
     def get_resume_pos(self, doc, pos):
         if pos == 0:
             return pos
 
-        # check a character proceeding to updated pos
-        pos -= 1
-        style = doc.styles.getints(pos, pos + 1)[0]
-        if style == 0:
-            # not highlighted yet.
-            p = doc.styles.rfindint([0], 0, pos, comp_ne=True)
-            if p != -1:
-                return p + 1
-            return 0
+#        # check a character proceeding to updated pos
+#        pos -= 1
+#        style = doc.styles.getints(pos, pos + 1)[0]
+#        if style == 0:
+#            # not highlighted yet.
+#            p = doc.styles.rfindint([0], 0, pos, comp_ne=True)
+#            if p != -1:
+#                return p + 1
+#            return 0
+#
+#        pair = self.tokenids.get(style)
+#        if not pair:
+#            # Invalid. this style is not set by tokenizer.
+#            return 0
+#
+#        tokenizer, token = self.tokenids.get(style)
+#        if not token:
+#            # token is not defined here. (e.g. white spaces)
+#            # resume at beggining of this non-defined area.
+#            p = doc.styles.rfindint([style], 0, pos, comp_ne=True)
+#            if p != -1:
+#                return p + 1
+#            return 0
+#        return token.resume_pos(self, tokenizer, doc, pos)
 
-        pair = self.tokenids.get(style)
-        if not pair:
-            # Invalid. this style is not set by tokenizer.
-            return 0
-
-        tokenizer, token = self.tokenids.get(style)
+        tokenizer, token, pos = self.get_prev_token(doc, pos)
         if not token:
-            p = doc.styles.rfindint([style], 0, pos, comp_ne=True)
-            if p != -1:
-                return p + 1
             return 0
 
+        # Let the token at the pos determin resume pos.
         return token.resume_pos(self, tokenizer, doc, pos)
 
     def updated(self, doc, pos, inslen, dellen):
