@@ -3,7 +3,6 @@ import os
 import fnmatch
 import re
 
-from gappedbuf import re as gre
 import kaa
 from kaa import document
 from kaa.theme import Theme, Style
@@ -207,8 +206,7 @@ def _search(dir, option, doc):
 def grep(option, target):
 
     if not target:
-        buf = document.Buffer()
-        doc = document.Document(buf)
+        doc = document.Document()
         mode = GrepMode()
         doc.setmode(mode)
     else:
