@@ -28,8 +28,7 @@ class ItemListMode(dialogmode.DialogMode):
 
     @classmethod
     def build(cls, caption, items, sel, callback, onchange=None):
-        buf = document.Buffer()
-        doc = document.Document(buf)
+        doc = document.Document()
         mode = cls()
         doc.setmode(mode)
 
@@ -82,7 +81,7 @@ class ItemListMode(dialogmode.DialogMode):
         self.cursel = None
         self.selected(wnd)
 
-    def on_str(self, wnd, s):
+    def on_str(self, wnd, s, overwrite=False):
         cursel = self.cursel
         if cursel is None:
             cursel = -1
