@@ -33,13 +33,13 @@ class MsgBoxMode(dialogmode.DialogMode):
 
         wnd.CURSOR_TO_MIDDLE_ON_SCROLL = False
 
-    def on_str(self, wnd, s):
+    def on_str(self, wnd, s, overwrite=False):
         pass
 
     def on_start(self, wnd):
         wnd.cursor.setpos(self.document.endpos() - 1)
 
-    def on_str(self, wnd, s):
+    def on_str(self, wnd, s, overwrite=False):
         for c in s:
             c = c.lower()
             if c in self.shortcuts:
@@ -74,8 +74,7 @@ class MsgBoxMode(dialogmode.DialogMode):
 
     @classmethod
     def build_msgbox(cls, caption, options, callback, keys=None, border=False):
-        buf = document.Buffer()
-        doc = document.Document(buf)
+        doc = document.Document()
         mode = cls()
         mode.callback = callback
         mode.keys = keys
